#!/bin/sh


#
# Copyright (C) 2017  Hiveon Holding LTD
# Distributed under Business Source License 1.1
# License information can be found in the LICENSE.txt file or at https://github.com/minershive/hiveos-asic/blob/master/LICENSE.txt
#
# Linted by shellcheck 0.7.0
# shellcheck shell=ash
# shellcheck disable=SC2009
# shellcheck disable=SC2169
#


readonly script_mission='Hiveon Firmware: rig.conf migration'
readonly script_description='Fetch $HIVE_HOST_URL from the agent-ng config and paste it to the rig.conf'
readonly script_version='0.0.1'
readonly script_basename="${0##*/}"


# functions from library

get_jq_path () {
	#
	# let's use fastest jq available (1.5)
	#

	# consts
	local fast_jq='/hive/sbin/jq'

	# code
	# shellcheck disable=SC2230 # we use "which" bcs in BusyBox v1.20.2 there's no 'command' builtin
	if [ -x "$fast_jq" ]; then
		echo "$fast_jq"
		return "$E_OK"
	elif which 'jq'; then
		# the "which" output already echoed
		return "$E_OK"
	else
		return "$E_FAIL"
	fi
}

set_variable_in_file () {
	#
	# Usage: set_variable_in_file 'file_with_variables' 'variable_to_change' 'new_value'
	#
	# if the variable isn't exist, add it to the end of file
	# if the variable is defined as empty, like 'var=', add a value to it

	# args
	local file_with_variables_ARG="$1"
	local variable_to_change_ARG="$2"
	local new_value_ARG="'$3'" # escape with quotes first thing

	# vars
	local empty_if_ends_with_newline

	# code
	# does the file exist and isn't empty?
	if [ -s "$file_with_variables_ARG" ]; then
		# yes, the file does exist and isn't empty
		# is the variable already there?
		if grep -Eq -e "^${variable_to_change_ARG}=.*\$" -- "$file_with_variables_ARG"; then
			# yes, change its value
			sed -i "s|^${variable_to_change_ARG}=.*\$|${variable_to_change_ARG}=${new_value_ARG}|" "$file_with_variables_ARG"
		else
			# no, add variable
			empty_if_ends_with_newline="$( tail -c 1 "$file_with_variables_ARG" )"
			{
				[ -n "$empty_if_ends_with_newline" ] && echo # add a newline first
				echo "${variable_to_change_ARG}=${new_value_ARG}"
			} >> "$file_with_variables_ARG"
		fi
	else
		# no, the file does not exist or it's empty
		# just write the variable
		echo "${variable_to_change_ARG}=${new_value_ARG}" >> "$file_with_variables_ARG"
	fi
}


# other functions

is_localhost_in_legacy_agent_config () {
	# asserts
	[ -s "$rig_conf_FILE" ]	|| return "$E_FAIL"
	. "$rig_conf_FILE"		|| return "$E_FAIL"
	[ -n "$HIVE_HOST_URL" ]	|| return "$E_FAIL"

	# result
	echo "$HIVE_HOST_URL" | grep -Eq '127.0.0.1|localhost'
}

fetch_first_url_from_agent_ng_config () {
	# vars
	local jq_app_FILE first_url

	# code
	# asserts
	[ -s "$agent_ng_conf_FILE" ]											|| return "$E_FAIL"
	jq_app_FILE="$( get_jq_path )"											|| return "$E_FAIL"
	first_url="$(
		"$jq_app_FILE" -e --raw-output '.api_urls[0]' "$agent_ng_conf_FILE"
	)"																		|| return "$E_FAIL"
	[ -n "$first_url" ]														|| return "$E_FAIL"

	# result
	echo "$first_url"
}

paste_agent_ng_url_to_legacy_agent_conf () {
	# vars
	local first_url

	# code
	first_url="$( fetch_first_url_from_agent_ng_config )"	|| return "$E_FAIL"
	[ -n "$first_url" ]										|| return "$E_FAIL"

	echo "Writing '${first_url}' into $rig_conf_FILE"
	set_variable_in_file "$rig_conf_FILE" 'HIVE_HOST_URL' "$first_url"
}


# global enums
readonly E_OK=0 E_FAIL=1 E_NOT_FOUND=2
# shellcheck disable=SC2034
readonly C_TRUE=1 C_FALSE=0

# global props
readonly rig_conf_FILE='/hive-config/rig.conf'
readonly agent_ng_conf_FILE='/config/agent-ng/config.json'


# main
echo "$script_mission, version $script_version"

echo 'Checking migration status'
if is_localhost_in_legacy_agent_config; then
	echo 'Gotcha! Starting migration'
else
	echo 'No need for migration'
	exit "$E_NOT_FOUND"
fi

if paste_agent_ng_url_to_legacy_agent_conf; then
	echo 'Finished'
else
	echo 'Something goes wrong'
	exit "$E_FAIL"
fi
